!{\src2tex{textfont=tt}}
!!****f* ABINIT/check/test_radsintr
!! NAME
!!  test_radsintr
!!
!! FUNCTION
!!  Tests the routine src/32_util/radsintr.F90 that performs 
!!  radial Fourier transform.
!!
!! COPYRIGHT
!!  Copyright (C) 2011 ABINIT group (CE)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  (main routine)
!!
!! OUTPUT
!!  (main routine)
!!
!! NOTES
!!  All required data are generated internally. This program
!!  performs radial FT of a gaussian function and then transforms  
!!  it back to real space to check possible differences. 
!!   
!! PARENTS
!!
!! CHILDREN
!!      init_radmesh,radsintr
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

program test_radsintr
    
 use defs_basis
 use defs_datatypes, only: pawrad_type
 use m_errors
 use m_profiling
 use m_radmesh, only: init_radmesh

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'test_radsintr'
 use interfaces_32_util
!End of the abilint section

 implicit none

!Arguments ------------------------------------

!Local variables-------------------------------
 integer :: ii,jj,mqgrid,mesh_size,mesh_type
 type(pawrad_type) :: qmesh,radmesh,funr_mesh
 real(dp) :: dq,lstep,rstep,r_for_intg,yq1,yqn,yr1,yrn   
 real(dp), allocatable :: funr(:),funr2(:)
 real(dp), allocatable :: funq(:)

!character(len=500) :: message                   ! to be uncommented, if needed
 
! *************************************************************************

!!  Parameters related to the generation of the radial meshes by
!!  init_radmesh:
!!
!!  mesh_size=Dimension of the radial mesh.
!!  mesh_type=Type of mesh
!!     1=regular grid: r(i)=(i-1)*AA
!!     2=logarithmic grid: r(i)=AA*(exp[BB*(i-1)]-1)
!!     3=logarithmic grid: r(i>1)=AA*exp[BB*(i-1)] and r(1)=0
!!     4=logarithmic grid: r(i)=-AA*ln[1-BB*(i-1)] with BB=1/n
!!  rstep=Radial step of the mesh (AA parameter above)
!!  lstep=Exponential step of the mesh (BB parameter above)
!!    Needed only if mesh type is logarithmic.
!!  r_for_intg=Mesh size used in integrals computation (<0 means whole interval)
!!    Integrals will be computed up to rr(r_for_intg).
!!
!*****************************************************************************

!Parameters for the generation of the grids
 mesh_size = 150
 mesh_type = 1       ! mesh_type,r_for_intg are common to
 r_for_intg=-1.0d0   ! real and reciprocal space meshes
 rstep = 0.02d0     

 mqgrid = 50       ! q mesh parameters
 dq=0.04

!Initialization of meshes  
 call init_radmesh(radmesh,mesh_size,mesh_type,rstep,lstep,r_for_intg)
 call init_radmesh(qmesh,mqgrid,mesh_type,dq,lstep,r_for_intg)

 ABI_ALLOCATE(funr,(radmesh%mesh_size))
 ABI_ALLOCATE(funq,(qmesh%mesh_size))
 ABI_ALLOCATE(funr2,(radmesh%mesh_size))
 
 do ii=1,radmesh%mesh_size
   funr(ii)=exp(-((radmesh%rad(ii)-one)**two)/0.1d0)
 end do

 call radsintr(funr,funq,qmesh%mesh_size,qmesh%rad(:),radmesh,yq1,yqn)
 
 write(std_out,*) ch10, 'r  F(r):',ch10
 do jj=1,radmesh%mesh_size
   write(std_out,*) radmesh%rad(jj), funr(jj)
 end do

 write(std_out,*) ch10, 'q  F(q):',ch10
 do ii=1,qmesh%mesh_size
   write(std_out,*) qmesh%rad(ii), funq(ii)  
 end do


 call radsintr(funq,funr2,radmesh%mesh_size,radmesh%rad(:),qmesh,yr1,yrn)

 write(std_out,*) ch10, 'r  F2(r):',ch10
 do ii=1,radmesh%mesh_size
   write(std_out,*) radmesh%rad(ii), funr2(ii)  
 end do

 ABI_DEALLOCATE(funq)
 ABI_DEALLOCATE(funr)
 ABI_DEALLOCATE(funr2)
 
!DEBUG
!write (std_out,*) ' test_radsintr : enter'
!ENDDEBUG

!DEBUG                                           ! to be uncommented, if needed
!if(option/=1 .and. option/=2 )then
!write(message,'(a,a,a,a,a,a,i6)') ch10,&
!&  ' test_radsintr: BUG -',ch10,&
!&  '  The argument option should be 1 or 2,',ch10,&
!&  '  however, option=',option
!call wrtout(std_out,message,'COLL')
!call leave_new('COLL')
!endif
!if(sizein<1)then
!write(message,'(a,a,a,a,a,a,i6)') ch10,&
!&  ' test_radsintr: BUG -',ch10,&
!&  '  The argument sizein should be a positive number,',ch10,&
!&  '  however, sizein=',sizein
!call wrtout(std_out,message,'COLL')
!call leave_new('COLL')
!endif
!ENDDEBUG


!DEBUG
!write (std_out,*) ' test_radsintr : exit'
!stop
!ENDDEBUG

 end program test_radsintr
!!***
