!{\src2tex{textfont=tt}}
!!****f* ABINIT/paw2wvl
!! NAME
!!  paw2wvl
!!
!! FUNCTION
!!  Points WVL objects to PAW objects
!!
!! COPYRIGHT
!!  Copyright (C) 2011 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  argin(sizein)=description
!!
!! OUTPUT
!!  argout(sizeout)=description
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      gstate
!!
!! CHILDREN
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine paw2wvl(indlmn,lmnmax,ntypat,pawtab,proj,wvl)
    
 use m_profiling
 use m_errors
 use defs_basis
 use defs_datatypes
 use defs_wvltypes
!#if defined HAVE_DFT_BIGDFT
!  use BigDFT_API, only: nullify_gaussian_basis
!#endif

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'paw2wvl'
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 integer,intent(in)::ntypat 
 integer,intent(in)::lmnmax
 integer,intent(in)::indlmn(6,lmnmax,ntypat)
 type(pawtab_type),intent(in)::pawtab(ntypat)
 type(wvl_internal_type), intent(inout) :: wvl
 type(wvl_projectors_type),intent(inout)::proj

!Local variables-------------------------------
 real(dp) :: a1
 integer :: ib,ig
 integer :: itypat,jb,ll,nn,ll_,nn_
 integer ::maxmsz,msz1,ptotgau,max_lmn2_size
 integer :: msz(ntypat)
 character(len=500) :: message                   ! to be uncommented, if needed

  !extra variables, use to debug
  !
  !integer::nr,unitp,i_shell,ii,ir,ng
  !real(dp)::step,rmax
  !real(dp),allocatable::r(:), y(:)
  !complex::fac,arg
  !complex(dp),allocatable::f(:),g(:,:)

! *********************************************************************


!DEBUG
!write (std_out,*) ' paw2wvl : enter'
!ENDDEBUG

#if defined HAVE_DFT_BIGDFT


!Find max mesh size
 do itypat=1,ntypat
   msz(itypat)=pawtab(itypat)%wvl%rholoc%msz
 end do
 maxmsz=maxval(msz(1:ntypat))

 ABI_ALLOCATE(wvl%rholoc%msz,(ntypat))
 ABI_ALLOCATE(wvl%rholoc%d,(maxmsz,4,ntypat))
 ABI_ALLOCATE(wvl%rholoc%rad,(maxmsz,ntypat))
 ABI_ALLOCATE(wvl%rholoc%radius,(ntypat))

 do itypat=1,ntypat
   msz1=pawtab(itypat)%wvl%rholoc%msz
   msz(itypat)=msz1
   wvl%rholoc%msz(itypat)=msz1
   wvl%rholoc%d(1:msz1,:,itypat)=pawtab(itypat)%wvl%rholoc%d(1:msz1,:)
   wvl%rholoc%rad(1:msz1,itypat)=pawtab(itypat)%wvl%rholoc%rad(1:msz1)
   wvl%rholoc%radius(itypat)=pawtab(itypat)%rpaw
 end do
!
!Now fill projectors type:
!
 ABI_DATATYPE_ALLOCATE(proj%G,(ntypat))
!
!nullify all for security:
!do itypat=1,ntypat
!call nullify_gaussian_basis(proj%G(itypat))
!end do
 proj%G(:)%nat=1 !not used
 proj%G(:)%ncplx=2 !Complex gaussians
!
!Obtain dimensions:
!jb=0
!ptotgau=0
 do itypat=1,ntypat
!  do ib=1,pawtab(itypat)%basis_size
!  jb=jb+1
!  end do
!  ptotgau=ptotgau+pawtab(itypat)%wvl%ptotgau
   ptotgau=pawtab(itypat)%wvl%ptotgau
   proj%G(itypat)%nexpo=ptotgau
   proj%G(itypat)%nshltot=pawtab(itypat)%basis_size
 end do
!proj%G%nexpo=ptotgau
!proj%G%nshltot=jb
!
!Allocations
 do itypat=1,ntypat
   ABI_ALLOCATE(proj%G(itypat)%ndoc ,(proj%G(itypat)%nshltot))
   ABI_ALLOCATE(proj%G(itypat)%nam  ,(proj%G(itypat)%nshltot))
   ABI_ALLOCATE(proj%G(itypat)%xp   ,(proj%G(itypat)%ncplx,proj%G(itypat)%nexpo))
   ABI_ALLOCATE(proj%G(itypat)%psiat,(proj%G(itypat)%ncplx,proj%G(itypat)%nexpo))
 end do

!jb=0
 do itypat=1,ntypat
   proj%G(itypat)%ndoc(:)=pawtab(itypat)%wvl%pngau(:)
 end do
!
 do itypat=1,ntypat
   proj%G(itypat)%xp(:,:)=pawtab(itypat)%wvl%parg(:,:)
   proj%G(itypat)%psiat(:,:)=pawtab(itypat)%wvl%pfac(:,:)
 end do

!Change the real part of psiat to the form adopted in BigDFT:
!Here we use exp{(a+ib)x^2}, where a is a negative number.
!In BigDFT: exp{-0.5(x/c)^2}exp{i(bx)^2}, and c is positive
!Hence c=sqrt(0.5/abs(a))
 do itypat=1,ntypat
   do ig=1,proj%G(itypat)%nexpo
     a1=proj%G(itypat)%xp(1,ig)
!    First check that "a" is a negative number:
     if( a1 > tol12) then
       message = 'Real part of Gaussians should be a negative number (they should go to zero at infty)'
       MSG_ERROR(message)
     end if
     a1=(sqrt(0.5/abs(a1)))
     proj%G(itypat)%xp(1,ig)=a1
   end do
 end do

!debug
!write(*,*)'paw2wvl 178: erase me set gaussians real equal to hgh (for Li)'
!ABI_DEALLOCATE(proj%G(1)%xp)
!ABI_DEALLOCATE(proj%G(1)%psiat)
!ABI_ALLOCATE(proj%G(1)%psiat,(2,2))
!ABI_ALLOCATE(proj%G(1)%xp,(2,2))
!proj%G(1)%ndoc(:)=1 !1 gaussian per shell
!proj%G(1)%nexpo=2 ! two gaussians in total
!proj%G(1)%xp=zero
!proj%G(1)%xp(1,1)=0.666375d0
!proj%G(1)%xp(1,2)=1.079306d0
!proj%G(1)%psiat(1,:)=1.d0
!proj%G(1)%psiat(2,:)=zero

!
!begin debug
!
!write(*,*)'paw2wvl, comment me'
!!and comment out variables 
!nr=3000
!rmax=10.d0
!ABI_ALLOCATE(r,(nr))
!ABI_ALLOCATE(f,(nr))
!ABI_ALLOCATE(y,(nr))
!step=rmax/real(nr-1,dp)
!do ir=1,nr
!r(ir)=real(ir-1,dp)*step
!end do
!
!unitp=400
!do itypat=1,ntypat
!do i_shell=1,proj%G(itypat)%nshltot
!unitp=unitp+1
!ig=0
!f(:)=czero
!ng=proj%G(itypat)%ndoc(i_shell)
!ABI_ALLOCATE(g,(nr,ng))
!do ii=1,ng
!!
!ig=ig+1
!fac=cmplx(proj%G(itypat)%psiat(1,ig),proj%G(itypat)%psiat(2,ig))
!a1=-0.5d0/(proj%G(itypat)%xp(1,ig)**2)
!arg=cmplx(a1,proj%G(itypat)%xp(2,ig))
!g(:,ig)=fac*exp(arg*r(:)**2)
!f(:)=f(:)+g(:,ii)
!end do
!do ir=1,nr
!write(unitp,'(9999f16.7)')r(ir),real(f(ir)),(real(g(ir,ii)),&
!&    ii=1,ng)
!end do
!ABI_DEALLOCATE(g)
!end do
!end do
!ABI_DEALLOCATE(r)
!ABI_DEALLOCATE(f)
!ABI_DEALLOCATE(y)
 
!end debug


!jb=0
 do itypat=1,ntypat
   jb=0
   do ib=1,pawtab(itypat)%lmn_size 
     ll=indlmn(1,ib,itypat)
     nn=indlmn(3,ib,itypat)
!    write(*,*)ll,indlmn(2,ib,itypat),nn
     if(ib>1 .and. ll == ll_ .and. nn==nn_) cycle
     jb=jb+1
!    proj%G%nam(jb)= indlmn(1,ib,itypat)+1!l quantum number
!    write(*,*)jb,proj%G%nshltot,proj%G%nam(jb)
     proj%G(itypat)%nam(jb)= indlmn(1,ib,itypat)+1 !l quantum number
!    1 is added due to BigDFT convention
!    write(*,*)jb,indlmn(1:3,ib,itypat)
     ll_=ll
     nn_=nn
   end do
 end do
!Nullify remaining objects
 do itypat=1,ntypat
   nullify(proj%G(itypat)%rxyz)
   nullify(proj%G(itypat)%nshell)
 end do
!nullify(proj%G%rxyz)

!now the index l,m,n objects:
 ABI_ALLOCATE(wvl%paw%indlmn,(6,lmnmax,ntypat))
 wvl%paw%indlmn=indlmn
 wvl%paw%lmnmax=lmnmax
 wvl%paw%ntypes=ntypat

!allocate and copy sij
!max_lmn2_size=max(pawtab(:)%lmn2_size,1)
 max_lmn2_size=lmnmax*(lmnmax+1)/2
!
 ABI_ALLOCATE(wvl%paw%sij,(max_lmn2_size,ntypat))
!sij is not yet calculated here.
!We copy this in gstate after call to pawinit
!do itypat=1,ntypat
!wvl%paw%sij(1:pawtab(itypat)%lmn2_size,itypat)=pawtab(itypat)%sij(:)
!end do
 
#endif

!DEBUG
!write (std_out,*) ' paw2wvl : exit'
!stop
!ENDDEBUG

end subroutine paw2wvl
!!***
