!{\src2tex{textfont=tt}}
!!****f* ABINIT/pawrad_bcast
!! NAME
!! pawrad_bcast
!!
!! FUNCTION
!! Communicate pawrad data to all processors
!!
!! COPYRIGHT
!! Copyright (C) 2009-2013 ABINIT group (FJ,MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt.
!!
!! INPUTS
!! comm_mpi= communicator used to broadcast data
!!
!! SIDE EFFECTS
!!  pawrad=<type pawrad_type>
!!   all the important information from the
!!   pseudopotential file. On one processor at input,
!!   on all processors at output
!!
!! PARENTS
!!      pawbcast
!!
!! CHILDREN
!!      leave_test,timab,xcast_mpi
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"


subroutine pawrad_bcast(comm_mpi,pawrad)

 use m_profiling

 use defs_basis
 use defs_datatypes
 use m_xmpi
 use m_errors
#if defined HAVE_MPI2
 use mpi
#endif

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'pawrad_bcast'
 use interfaces_18_timing
 use interfaces_51_manage_mpi, except_this_one => pawrad_bcast
!End of the abilint section

 implicit none
#if defined HAVE_MPI1
 include 'mpif.h'
#endif

!Arguments ------------------------------------
 integer,intent(in) :: comm_mpi
 type(pawrad_type),intent(inout) :: pawrad

!Local variables-------------------------------

!Variables introduced for MPI version
 integer :: ierr,indx,me,nn,isz1
 integer,allocatable :: list_int(:)
 real(dp) :: tsec(2)
 real(dp),allocatable :: list_dpr(:)
 character(len=500) :: message

!*************************************************************************

 call timab(48,1,tsec)
 call leave_test()
 me=xcomm_rank(comm_mpi)
!calculate the size of the reals
 if(me==0) then
   nn=4
   if (associated(pawrad%rad)) then
     nn=nn+size(pawrad%rad)
   end if
   if (associated(pawrad%radfact)) then
     nn=nn+size(pawrad%radfact)
   end if
   if (associated(pawrad%simfact)) then
     nn=nn+size(pawrad%simfact)
   end if
 end if

!Brodcast the integers
 ABI_ALLOCATE(list_int,(4))
 if(me==0) then
   list_int(1)=pawrad%int_meshsz
   list_int(2)=pawrad%mesh_size
   list_int(3)=pawrad%mesh_type
   list_int(4)=nn
 end if

 call xcast_mpi(list_int,0,comm_mpi,ierr)

 if(me/=0) then
   pawrad%int_meshsz =list_int(1)
   pawrad%mesh_size =list_int(2)
   pawrad%mesh_type =list_int(3)
   nn=list_int(4)
 end if
 ABI_DEALLOCATE(list_int)

!Broadcast the reals
 ABI_ALLOCATE(list_dpr,(nn))
 if(me==0) then
   list_dpr(1)=pawrad%lstep
   list_dpr(2)=pawrad%rmax
   list_dpr(3)=pawrad%rstep
   list_dpr(4)=pawrad%stepint
   indx=5
   if (associated(pawrad%rad)) then
     isz1=size(pawrad%rad)
     if(isz1/=pawrad%mesh_size) then
       write(message, '(a,a,a)' )&
&       '  pawrad_bcast: rad: sz1 /= pawrad%mesh_size ',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     list_dpr(indx:indx+isz1-1)=pawrad%rad(:)
     indx=indx+isz1
   end if
   if (associated(pawrad%radfact)) then
     isz1=size(pawrad%radfact)
     if(isz1/=pawrad%mesh_size) then
       write(message, '(a,a,a)' )&
&       '  pawrad_bcast:radfact: sz1 /= pawrad%mesh_size ',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     list_dpr(indx:indx+isz1-1)=pawrad%radfact(:)
     indx=indx+isz1
   end if
   if (associated(pawrad%simfact)) then
     isz1=size(pawrad%simfact)
     if(isz1/=pawrad%mesh_size) then
       write(message, '(a,a,a)' )&
&       '  pawrad_bcast: simfact: sz1 /= pawrad%mesh_size ',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     list_dpr(indx:indx+isz1-1)=pawrad%simfact(:)
     indx=indx+isz1
   end if
 end if

 call xcast_mpi(list_dpr,0,comm_mpi,ierr)

 if(me/=0) then
   pawrad%lstep=list_dpr(1)
   pawrad%rmax=list_dpr(2)
   pawrad%rstep=list_dpr(3)
   pawrad%stepint=list_dpr(4)
   indx=5
   if (associated(pawrad%rad)) then
     ABI_DEALLOCATE(pawrad%rad)
     isz1=pawrad%mesh_size
     ABI_ALLOCATE(pawrad%rad,(isz1))
     pawrad%rad(:)=list_dpr(indx:indx+isz1-1)
     indx=indx+isz1
   end if
   if (associated(pawrad%radfact)) then
     ABI_DEALLOCATE(pawrad%radfact)
     isz1=pawrad%mesh_size
     ABI_ALLOCATE(pawrad%radfact,(isz1))
     pawrad%radfact(:)=list_dpr(indx:indx+isz1-1)
     indx=indx+isz1
   end if
   if (associated(pawrad%simfact)) then
     ABI_DEALLOCATE(pawrad%simfact)
     isz1=pawrad%mesh_size
     ABI_ALLOCATE(pawrad%simfact,(isz1))
     pawrad%simfact(:)=list_dpr(indx:indx+isz1-1)
     indx=indx+isz1
   end if
 end if
 ABI_DEALLOCATE(list_dpr)
 call timab(48,2,tsec)

end subroutine pawrad_bcast
!!***
