!{\src2tex{textfont=tt}}
!!****f* ABINIT/pawtab_bcast
!! NAME
!! pawtab_bcast
!!
!! FUNCTION
!! Communicate pawtab data to all processors
!!
!! COPYRIGHT
!! Copyright (C) 2009-2013 ABINIT group (FJ,MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt.
!!
!! INPUTS
!! comm_mpi= communicator used to broadcast data
!!
!! SIDE EFFECTS
!!  pawtab=<type pawtab_type>
!!   all the important information from the
!!   pseudopotential file. On one processor at input,
!!   on all processors at output
!!
!! PARENTS
!!      pawbcast
!!
!! CHILDREN
!!      leave_test,timab,xcast_mpi
!!
!! NOTES
!!      Only data calculated in psp7in or psp17in are broadcasted
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"


subroutine pawtab_bcast(comm_mpi,pawtab)

 use m_profiling

 use defs_basis
 use defs_datatypes
 use m_xmpi
 use m_errors
#if defined HAVE_MPI2
 use mpi
#endif

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'pawtab_bcast'
 use interfaces_18_timing
 use interfaces_51_manage_mpi, except_this_one => pawtab_bcast
!End of the abilint section

 implicit none
#if defined HAVE_MPI1
 include 'mpif.h'
#endif

!Arguments ------------------------------------
 integer,intent(in) :: comm_mpi
 type(pawtab_type),intent(inout) :: pawtab

!Local variables-------------------------------


!Variables introduced for MPI version
 integer :: ierr,ii,indx,me,nn,nn_scalars,isz1
 integer :: isz1_tvalespl
 integer :: isz2,isz2_tcoredens,isz2_rholoc_d,isz2_tvalespl
 integer,allocatable :: list_int(:)
 real(dp) :: tsec(2)
 real(dp),allocatable :: list_dpr(:)
 character (len=500) :: message


!*************************************************************************

 call timab(48,1,tsec)
 call leave_test()
 me=xcomm_rank(comm_mpi)
 nn_scalars=7 !number of scalar variables passed in list_dpr: see below

!calculate the size of the reals
 if(me==0) then
   nn=nn_scalars 
   if (associated(pawtab%coredens)) then
     nn=nn+size(pawtab%coredens)
   end if
   isz2_tcoredens=0
   if (associated(pawtab%tcoredens)) then
     nn=nn+size(pawtab%tcoredens)
     isz2_tcoredens=size(pawtab%tcoredens,2)
   end if
   if (associated(pawtab%rhoij0)) then
     nn=nn+size(pawtab%rhoij0)
   end if
   if (associated(pawtab%dij0)) then
     nn=nn+size(pawtab%dij0)
   end if
   if (associated(pawtab%kij)) then
     nn=nn+size(pawtab%kij)
   end if
   if (associated(pawtab%vhntzc)) then
     nn=nn+size(pawtab%vhntzc)
   end if
   if (associated(pawtab%vhnzc)) then
     nn=nn+size(pawtab%vhnzc)
   end if
   if (associated(pawtab%shape_alpha)) then
     nn=nn+size(pawtab%shape_alpha)
   end if
   if (associated(pawtab%shape_q)) then
     nn=nn+size(pawtab%shape_q)
   end if
   if (associated(pawtab%shapefunc)) then
     nn=nn+size(pawtab%shapefunc)
   end if
   if (associated(pawtab%phi)) then
     nn=nn+size(pawtab%phi)
   end if
   if (associated(pawtab%tphi)) then
     nn=nn+size(pawtab%tphi)
   end if
   if (associated(pawtab%tcorespl)) then
     nn=nn+size(pawtab%tcorespl)
   end if
   isz1_tvalespl=0 ; isz2_tvalespl=0
   if (associated(pawtab%tvalespl)) then
     nn=nn+size(pawtab%tvalespl)
     isz1_tvalespl=size(pawtab%tvalespl,1)
     isz2_tvalespl=size(pawtab%tvalespl,2)
   end if
   isz2_rholoc_d=0
   if (pawtab%wvl%rholoc%msz>0) then
     if (associated(pawtab%wvl%rholoc%d)) then
       nn=nn+size(pawtab%wvl%rholoc%d)
       isz2_rholoc_d=size(pawtab%wvl%rholoc%d,2)
     end if
     if (associated(pawtab%wvl%rholoc%rad)) then
       nn=nn+size(pawtab%wvl%rholoc%rad)
     end if
   end if
 end if

!Brodcast the integers
 ABI_ALLOCATE(list_int,(20))
 if(me==0) then
   list_int(1)=pawtab%basis_size
   list_int(2)=pawtab%lmn_size
   list_int(3)=pawtab%l_size
   list_int(4)=pawtab%lmn2_size
   list_int(5)=pawtab%shape_type
   list_int(6)=pawtab%shape_lambda
   list_int(7)=pawtab%has_vhnzc
   list_int(8)=pawtab%usetcore
   list_int(9)=pawtab%usexcnhat
   list_int(10)=pawtab%usetvale
   list_int(11)=pawtab%has_vhntzc
   list_int(12)=pawtab%mqgrid
   list_int(13)=pawtab%has_kij
   list_int(14)=pawtab%core_mesh_size
   list_int(15)=pawtab%wvl%rholoc%msz
   list_int(16)=isz2_tcoredens
   list_int(17)=isz1_tvalespl
   list_int(18)=isz2_tvalespl
   list_int(19)=isz2_rholoc_d
   list_int(20)=nn
 end if

 call xcast_mpi(list_int,0,comm_mpi,ierr)

 if(me/=0) then
   pawtab%basis_size=list_int(1)
   pawtab%lmn_size=list_int(2)
   pawtab%l_size=list_int(3)
   pawtab%lmn2_size=list_int(4)
   pawtab%shape_type=list_int(5)
   pawtab%shape_lambda=list_int(6)
   pawtab%has_vhnzc=list_int(7)
   pawtab%usetcore=list_int(8)
   pawtab%usexcnhat=list_int(9)
   pawtab%usetvale=list_int(10)
   pawtab%has_vhntzc=list_int(11)
   pawtab%mqgrid=list_int(12)
   pawtab%has_kij=list_int(13)
   pawtab%core_mesh_size=list_int(14)
   pawtab%wvl%rholoc%msz=list_int(15)
   isz2_tcoredens=list_int(16)
   isz1_tvalespl=list_int(17)
   isz2_tvalespl=list_int(18)
   isz2_rholoc_d=list_int(19)
   nn=list_int(20)
 end if
 ABI_DEALLOCATE(list_int)

!Broadcast the reals
 ABI_ALLOCATE(list_dpr,(nn))
 if(me==0) then
   list_dpr(1)=pawtab%shape_sigma
   list_dpr(2)=pawtab%rshp
   list_dpr(3)=pawtab%rpaw
   list_dpr(4)=pawtab%dncdq0
   list_dpr(5)=pawtab%dnvdq0
   list_dpr(6)=pawtab%exccore
   list_dpr(7)=pawtab%rcore
   indx=nn_scalars+1
   if (associated(pawtab%coredens)) then
     isz1=size(pawtab%coredens)
     if(isz1/=pawtab%core_mesh_size) then
       write(message, '(a,a,a)' )&
&       '  pawtab_bcast: coredens: sz1 /= pawtab%core_mesh_size ',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     list_dpr(indx:indx+isz1-1)=pawtab%coredens(:)
     indx=indx+isz1
   end if
   if (associated(pawtab%tcoredens)) then
     isz1=size(pawtab%tcoredens,1)
     if(isz1/=pawtab%core_mesh_size) then
       write(message, '(a,a,a)' )&
&       '  pawtab_bcast: tcoredens: sz1 /= pawtab%core_mesh_size ',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     if(isz2_tcoredens /=1 .and. isz2_tcoredens /=6) then
       write(message, '(a,a,a)' )&
&       '  pawtab_bcast: tcoredens: sz2 /= 1 and 6 ',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     do ii=1,isz2_tcoredens
       list_dpr(indx:indx+isz1-1)=pawtab%tcoredens(1:isz1,ii)
       indx=indx+isz1
     end do
   end if
   if (associated(pawtab%rhoij0)) then
     isz1=size(pawtab%rhoij0)
     if(isz1/=pawtab%lmn2_size) then
       write(message, '(a,a,a)' )&
&       '  pawtab_bcast: rhoij0: sz1 /= pawtab%lmn2_size ',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     list_dpr(indx:indx+isz1-1)=pawtab%rhoij0(:)
     indx=indx+isz1
   end if
   if (associated(pawtab%dij0)) then
     isz1=size(pawtab%dij0)
     if(isz1/=pawtab%lmn2_size) then
       write(message, '(a,a,a)' )&
&       '  pawtab_bcast: dij0: sz1 /= pawtab%lmn2_size ',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     list_dpr(indx:indx+isz1-1)=pawtab%dij0(:)
     indx=indx+isz1
   end if
   if (associated(pawtab%kij)) then
     isz1=size(pawtab%kij)
     if(isz1/=pawtab%lmn2_size) then
       write(message, '(a,a,a)' )&
&       '  pawtab_bcast: kij: sz1 /= pawtab%lmn2_size ',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     list_dpr(indx:indx+isz1-1)=pawtab%kij(:)
     indx=indx+isz1
   end if
   if (associated(pawtab%vhntzc)) then
     isz1=size(pawtab%vhntzc)
     if(isz1/=pawtab%mesh_size) then
       write(message, '(a,a,a)' )&
&       '  pawtab_bcast: vhntzc: sz1 /= pawtab%mesh_size ',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     list_dpr(indx:indx+isz1-1)=pawtab%vhntzc(:)
     indx=indx+isz1
   end if
   if (associated(pawtab%vhnzc)) then
     isz1=size(pawtab%vhnzc)
     if(isz1/=pawtab%mesh_size) then
       write(message, '(a,a,a)' )&
&       '  pawtab_bcast: vhnzc: sz1 /= pawtab%mesh_size ',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     list_dpr(indx:indx+isz1-1)=pawtab%vhnzc(:)
     indx=indx+isz1
   end if
   if (associated(pawtab%shape_alpha)) then
     isz1=size(pawtab%shape_alpha)
     if(isz1/=2*pawtab%l_size) then
       write(message, '(a,a,a)' )&
&       '  pawtab_bcast: shape_alpha: sz1 /= 2*pawtab%l_size ',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     list_dpr(indx:indx+isz1-1)=reshape(pawtab%shape_alpha(:,:),(/isz1/))
     indx=indx+isz1
   end if
   if (associated(pawtab%shape_q)) then
     isz1=size(pawtab%shape_q)
     if(isz1/=2*pawtab%l_size) then
       write(message, '(a,a,a)' )&
&       '  pawtab_bcast: shape_q: sz1 /= 2*pawtab%l_size ',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     list_dpr(indx:indx+isz1-1)=reshape(pawtab%shape_q(:,:),(/isz1/))
     indx=indx+isz1
   end if
   if (associated(pawtab%shapefunc)) then
     isz1=size(pawtab%shapefunc)
     if(isz1/=pawtab%mesh_size*pawtab%l_size) then
       write(message, '(a,a,a)' )&
&       '  pawtab_bcast: shapefunc: sz1 /= pawtab%mesh_size*pawtab%l_size ',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     list_dpr(indx:indx+isz1-1)=reshape(pawtab%shapefunc(:,:),(/isz1/))
     indx=indx+isz1
   end if
   if (associated(pawtab%phi)) then
     isz1=size(pawtab%phi)
     if(isz1/=pawtab%mesh_size*pawtab%basis_size) then
       write(message, '(a,a,a)' )&
&       '  pawtab_bcast: phi: sz1 /= pawtab%mesh_size*pawtab%basis_size ',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     list_dpr(indx:indx+isz1-1)=reshape(pawtab%phi(:,:),(/isz1/))
     indx=indx+isz1
   end if
   if (associated(pawtab%tphi)) then
     isz1=size(pawtab%tphi)
     if(isz1/=pawtab%mesh_size*pawtab%basis_size) then
       write(message, '(a,a,a)' )&
&       '  pawtab_bcast: tphi: sz1 /= pawtab%mesh_size*pawtab%basis_size ',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     list_dpr(indx:indx+isz1-1)=reshape(pawtab%tphi(:,:),(/isz1/))
     indx=indx+isz1
   end if
   if (associated(pawtab%tcorespl)) then
     isz1=size(pawtab%tcorespl)
     if(isz1/=2*pawtab%mqgrid) then
       write(message, '(a,a,a)' )&
&       '  pawtab_bcast: tcorespl: sz1 /= 2*pawtab%mqgrid ',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     list_dpr(indx:indx+isz1-1)=reshape(pawtab%tcorespl(:,:),(/isz1/))
     indx=indx+isz1
   end if
!  tvalespl:
   if (associated(pawtab%tvalespl)) then
     if(isz2_tvalespl /=1 .and. isz2_tvalespl /=2) then
       write(message, '(a,a,a)' )&
&       '  pawtab_bcast: tvalespl: sz2 /= 1 and 2 ',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     do ii=1,isz2_tvalespl
       list_dpr(indx:indx+isz1_tvalespl-1)=pawtab%tvalespl(1:isz1_tvalespl,ii)
       indx=indx+isz1_tvalespl
     end do
   end if
!  rholoc:
   if(pawtab%wvl%rholoc%msz<0) then
     write(message, '(a,a,a)' )&
&     '  pawtab_bcast: pawtab%wvl%rholoc%msz < 0 ',ch10,&
&     '  BUG: contact ABINIT group.'
     MSG_ERROR(message)
   end if
   if(pawtab%wvl%rholoc%msz>0) then
     if(.not. associated(pawtab%wvl%rholoc%d) .or. &
&     .not. associated(pawtab%wvl%rholoc%rad)) then
       write(message, '(a,a,a)' )&
&       '  pawtab_bcast: rholoc%msz>0 and pawtab%wvl%rholoc not associated',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     if( size(pawtab%wvl%rholoc%rad,1) .ne. pawtab%wvl%rholoc%msz .or.&
&     size(pawtab%wvl%rholoc%d,1) .ne. pawtab%wvl%rholoc%msz ) then
       write(message, '(a,a,a)' )&
&       '  pawtab_bcast: wrong size in pawtab%wvl%rholoc',ch10,&
&       '  BUG: contact ABINIT group.'
       MSG_ERROR(message)
     end if
     isz1=pawtab%wvl%rholoc%msz
     do ii=1,isz2_rholoc_d
       list_dpr(indx:indx+isz1-1)=pawtab%wvl%rholoc%d(1:isz1,ii)
       indx=indx+isz1
     end do
     list_dpr(indx:indx+isz1-1)=pawtab%wvl%rholoc%rad(1:isz1)
     indx=indx+isz1
   end if
 end if

 call xcast_mpi(list_dpr,0,comm_mpi,ierr)

 if(me/=0) then
   pawtab%shape_sigma=list_dpr(1)
   pawtab%rshp       =list_dpr(2)
   pawtab%rpaw       =list_dpr(3)
   pawtab%dncdq0     =list_dpr(4)
   pawtab%dnvdq0     =list_dpr(5)
   pawtab%exccore    =list_dpr(6)
   pawtab%rcore      =list_dpr(7)
   indx=nn_scalars+1
   if (associated(pawtab%coredens)) then
     ABI_DEALLOCATE(pawtab%coredens)
     isz1=pawtab%core_mesh_size
     ABI_ALLOCATE(pawtab%coredens,(isz1))
     pawtab%coredens(:)=list_dpr(indx:indx+isz1-1)
     indx=indx+isz1
   end if
!  tcoredens:
   if (associated(pawtab%tcoredens)) then
     ABI_DEALLOCATE(pawtab%tcoredens)
   end if
   if(isz2_tcoredens>0) then
     isz1=pawtab%core_mesh_size
     ABI_ALLOCATE(pawtab%tcoredens,(isz1,isz2_tcoredens))
     do ii=1,isz2_tcoredens
       pawtab%tcoredens(:,ii)=list_dpr(indx:indx+isz1-1)
       indx=indx+isz1
     end do
   end if
   if (associated(pawtab%rhoij0)) then
     ABI_DEALLOCATE(pawtab%rhoij0)
     isz1=pawtab%lmn2_size
     ABI_ALLOCATE(pawtab%rhoij0,(isz1))
     pawtab%rhoij0(:)=list_dpr(indx:indx+isz1-1)
     indx=indx+isz1
   end if
   if (associated(pawtab%dij0)) then
     ABI_DEALLOCATE(pawtab%dij0)
     isz1=pawtab%lmn2_size
     ABI_ALLOCATE(pawtab%dij0,(isz1))
     pawtab%dij0(:)=list_dpr(indx:indx+isz1-1)
     indx=indx+isz1
   end if
   if (associated(pawtab%kij)) then
     ABI_DEALLOCATE(pawtab%kij)
     isz1=pawtab%lmn2_size
     ABI_ALLOCATE(pawtab%kij,(isz1))
     pawtab%kij(:)=list_dpr(indx:indx+isz1-1)
     indx=indx+isz1
   end if
   if (associated(pawtab%vhntzc)) then
     ABI_DEALLOCATE(pawtab%vhntzc)
     isz1=pawtab%mesh_size
     ABI_ALLOCATE(pawtab%vhntzc,(isz1))
     pawtab%vhntzc(:)=list_dpr(indx:indx+isz1-1)
     indx=indx+isz1
   end if
   if (associated(pawtab%vhnzc)) then
     ABI_DEALLOCATE(pawtab%vhnzc)
     isz1=pawtab%mesh_size
     ABI_ALLOCATE(pawtab%vhnzc,(isz1))
     pawtab%vhnzc(:)=list_dpr(indx:indx+isz1-1)
     indx=indx+isz1
   end if
   if (associated(pawtab%shape_alpha)) then
     ABI_DEALLOCATE(pawtab%shape_alpha)
     isz1=pawtab%l_size
     ABI_ALLOCATE(pawtab%shape_alpha,(2,isz1))
     pawtab%shape_alpha(:,:)=reshape(list_dpr(indx:indx+2*isz1-1),(/2,isz1/))
     indx=indx+2*isz1
   end if
   if (associated(pawtab%shape_q)) then
     ABI_DEALLOCATE(pawtab%shape_q)
     isz1=pawtab%l_size
     ABI_ALLOCATE(pawtab%shape_q,(2,isz1))
     pawtab%shape_q(:,:)=reshape(list_dpr(indx:indx+2*isz1-1),(/2,isz1/))
     indx=indx+2*isz1
   end if
   if (associated(pawtab%shapefunc)) then
     ABI_DEALLOCATE(pawtab%shapefunc)
     isz1=pawtab%mesh_size
     isz2=pawtab%l_size
     ABI_ALLOCATE(pawtab%shapefunc,(isz1,isz2))
     pawtab%shapefunc(:,:)=reshape(list_dpr(indx:indx+isz1*isz2-1),(/isz1,isz2/))
     indx=indx+isz1*isz2
   end if
   if (associated(pawtab%phi)) then
     ABI_DEALLOCATE(pawtab%phi)
     isz1=pawtab%mesh_size
     isz2=pawtab%basis_size
     ABI_ALLOCATE(pawtab%phi,(isz1,isz2))
     pawtab%phi(:,:)=reshape(list_dpr(indx:indx+isz1*isz2-1),(/isz1,isz2/))
     indx=indx+isz1*isz2
   end if
   if (associated(pawtab%tphi)) then
     ABI_DEALLOCATE(pawtab%tphi)
     isz1=pawtab%mesh_size
     isz2=pawtab%basis_size
     ABI_ALLOCATE(pawtab%tphi,(isz1,isz2))
     pawtab%tphi(:,:)=reshape(list_dpr(indx:indx+isz1*isz2-1),(/isz1,isz2/))
     indx=indx+isz1*isz2
   end if
   if (associated(pawtab%tcorespl)) then
     ABI_DEALLOCATE(pawtab%tcorespl)
     isz1=pawtab%mqgrid
     ABI_ALLOCATE(pawtab%tcorespl,(isz1,2))
     pawtab%tcorespl(:,:)=reshape(list_dpr(indx:indx+2*isz1-1),(/isz1,2/))
     indx=indx+2*isz1
   end if
!  tvalespl:
   if (associated(pawtab%tvalespl)) then
     ABI_DEALLOCATE(pawtab%tvalespl)
   end if
   if(isz2_tvalespl>0) then
     ABI_ALLOCATE(pawtab%tvalespl,(isz1_tvalespl,isz2_tvalespl))
     do ii=1,isz2_tvalespl
       pawtab%tvalespl(1:isz1_tvalespl,ii)=list_dpr(indx:indx+isz1_tvalespl-1)
       indx=indx+isz1_tvalespl
     end do
   end if
!  rholoc:
   if(pawtab%wvl%rholoc%msz>0) then
     if(associated(pawtab%wvl%rholoc%d)) then
       ABI_DEALLOCATE(pawtab%wvl%rholoc%d)
     end if
     if(associated(pawtab%wvl%rholoc%rad)) then
       ABI_DEALLOCATE(pawtab%wvl%rholoc%rad)
     end if
     isz1=pawtab%wvl%rholoc%msz
     ABI_ALLOCATE(pawtab%wvl%rholoc%d,(isz1,isz2_rholoc_d))
     ABI_ALLOCATE(pawtab%wvl%rholoc%rad,(isz1))
     do ii=1,isz2_rholoc_d
       pawtab%wvl%rholoc%d(1:isz1,ii)=&
&       list_dpr(indx:indx+isz1-1)
       indx=indx+isz1
     end do
     pawtab%wvl%rholoc%rad(1:isz1)=&
&     list_dpr(indx:indx+isz1-1)
     indx=indx+isz1
   end if
 end if
 ABI_DEALLOCATE(list_dpr)
 call timab(48,2,tsec)

end subroutine pawtab_bcast
!!***
