!{\src2tex{textfont=tt}}
!!****f* ABINIT/handle_ncerr
!! NAME
!! handle_ncerr
!!
!! FUNCTION
!! Rudimentary Error catching for NetCDF using routines: prints out a string
!! describing what was being done when the error code showed up.
!!
!! COPYRIGHT
!! Copyright (C) 2004-2013 ABINIT group (Mver)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt.
!!
!! INPUTS
!!   ncerr = error code. If different from the preset nf90_noerr, then error
!!   message = subroutine specified message (description of action which
!!    produced the error)
!!  [file]=name of the f90 file containing the caller
!!  [line]=line number of the file where problem occurred
!!
!! OUTPUT
!!  (only writes)
!!
!! PARENTS
!!      ab_define_var,m_bse_io,outxfhist,read_md_hist,write_eig,write_md_hist
!!      wrt_moldyn_netcdf
!!
!! CHILDREN
!!      int2char
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine handle_ncerr(ncerr,message,file,line)

 use defs_basis
 use m_profiling
 use m_errors
#ifdef HAVE_TRIO_NETCDF
 use netcdf
#endif

 use m_fstrings,        only : basename

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'handle_ncerr'
 use interfaces_27_toolbox_oop
!End of the abilint section

 implicit none

!Arguments --------------------------
 integer         ,intent(in) :: ncerr
 integer,optional,intent(in) :: line
 character(len=*),intent(in) :: message
 character(len=*),optional,intent(in) :: file

!local variables
 integer :: f90line
 character(len=10) :: lnum
 character(len=500) :: msg,f90name

! *********************************************************************

 if (PRESENT(line)) then
   f90line=line
 else 
   f90line=0
 end if
 call int2char(f90line,lnum)
 
 if (PRESENT(file)) then 
   f90name = basename(file)
 else
   f90name='Subroutine Unknown'
 end if

#ifdef HAVE_TRIO_NETCDF
 if (ncerr /= NF90_NOERR) then
   msg = 'Error in netcdf call while: '//trim(message)//trim(nf90_strerror(ncerr))
   MSG_ERROR(msg)
 end if
#else
 msg = ABI_FUNC//' NETCDF not defined at compile time'
 MSG_ERROR(msg)
#endif

end subroutine handle_ncerr
!!***
