!{\src2tex{textfont=tt}}
!!****f* ABINIT/psp7cc_wvl
!! NAME
!!  psp7cc_wvl
!!
!! FUNCTION
!!  Compute the first to the 5th derivatives of
!!  the core density
!!
!! COPYRIGHT
!!  Copyright (C) 2011-2013 ABINIT group (T. Rangel)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!
!! OUTPUT
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      psp7in
!!
!! CHILDREN
!!      nderiv_gen,smooth,spline
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

subroutine psp7cc_wvl(mesh,tcoredens)
    
 use defs_basis
 use m_splines
 use defs_datatypes
 use m_radmesh,          only : nderiv_gen

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'psp7cc_wvl'
 use interfaces_28_numeric_noabirule
!End of the abilint section

 implicit none

!Arguments ------------------------------------
 type(pawrad_type),intent(in) :: mesh
 real(dp),intent(inout) :: tcoredens(mesh%mesh_size,6)

!Local variables-------------------------------
!character(len=500) :: message                   ! to be uncommented, if needed
 integer,parameter::it=1 !number of steps for smoothing function
 logical,parameter::use_smooth=.true.
 
! *************************************************************************
 
!DEBUG
!write (std_out,*) ' psp7cc_wvl : enter'
!ENDDEBUG

!DEBUG                                           ! to be uncommented, if needed
!if(option/=1 .and. option/=2 )then
!write(message,'(a,a,a,a,a,a,i6)') ch10,&
!&  ' psp7cc_wvl: BUG -',ch10,&
!&  '  The argument option should be 1 or 2,',ch10,&
!&  '  however, option=',option
!call wrtout(std_out,message,'COLL')
!call leave_new('COLL')
!endif
!if(sizein<1)then
!write(message,'(a,a,a,a,a,a,i6)') ch10,&
!&  ' psp7cc_wvl: BUG -',ch10,&
!&  '  The argument sizein should be a positive number,',ch10,&
!&  '  however, sizein=',sizein
!call wrtout(std_out,message,'COLL')
!call leave_new('COLL')
!endif
!ENDDEBUG

!calculate first derivative from density, 
!and store it
 call nderiv_gen(tcoredens(:,2),tcoredens(:,1),1,mesh)

!get second derivative from density, and store it
 call spline(mesh%rad,tcoredens(:,1),mesh%mesh_size,&
& zero,zero,tcoredens(:,3))

!smooth functions, to avoid numerical instabilities
 if(use_smooth) then
   call smooth(tcoredens(:,2),mesh%mesh_size,it)
   call smooth(tcoredens(:,3),mesh%mesh_size,it)
 end if

!get third derivative from first derivative:
 call spline(mesh%rad,tcoredens(:,2),mesh%mesh_size,&
& zero,zero,tcoredens(:,4))


!get fourth derivative from second derivative:
 call spline(mesh%rad,tcoredens(:,3),mesh%mesh_size,&
& zero,zero,tcoredens(:,5))

!smooth 3rd and 4th order derivatives
 if(use_smooth) then
   call smooth(tcoredens(:,4),mesh%mesh_size,it)
   call smooth(tcoredens(:,5),mesh%mesh_size,it)
 end if


!get fifth derivative from third derivative: 
 call spline(mesh%rad,tcoredens(:,4),mesh%mesh_size,&
& zero,zero,tcoredens(:,6))


!smooth 5th order derivative
 if(use_smooth) then
   call smooth(tcoredens(:,6),mesh%mesh_size,it)
 end if

!open(unit=400,file='tcoredens.dat',form='formatted',status='unknown')
!do i=1,mesh%mesh_size
!write(400,'(10(f20.7,1x))')mesh%rad(i),tcoredens(i,:)
!end do
!close(400)

!call smooth(gg2,n1xccc,10)

!DEBUG
!write (std_out,*) ' psp7cc_wvl : exit'
!stop
!ENDDEBUG

end subroutine psp7cc_wvl
!!***
