!{\src2tex{textfont=tt}}
!!****f* ABINIT/wvl_nhatgrid
!! NAME
!! wvl_nhatgrid
!!
!! FUNCTION
!! Determine parts of the rectangular (fine) grid that are contained
!! inside spheres around atoms (used to compute n_hat density).
!! If corresponding option is selected, compute also g_l(r)*Y_lm(r)
!! (and derivatives) on this grid (g_l=radial shape function).
!!
!! COPYRIGHT
!! Copyright (C) 2011-2013 ABINIT group (T Rangel)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~abinit/COPYING
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~abinit/doc/developers/contributors.txt.
!!
!! INPUTS
!!
!! OUTPUT
!!  pawfgrtab(natom)%ifftsph(nfgd)=FFT index (fine grid) of a points in paw spheres around each atom
!!  pawfgrtab(natom)%nfgd= number of (fine grid) FFT points in paw spheres around atoms
!!  if (optgr0==1)
!!    pawfgrtab(natom)%gylm(nfgd,l_size**2)= g_l(r)*Y_lm(r) around each atom
!!  if (optgr1==1)
!!    pawfgrtab(natom)%gylmgr(3,nfgd,l_size**2)= derivatives of g_l(r)*Y_lm(r) wrt cart. coordinates
!!  if (optgr2==1)
!!    pawfgrtab(natom)%gylmgr2(6,nfgd,l_size**2)= second derivatives of g_l(r)*Y_lm(r) wrt cart. coordinates
!!  if (optrad==1)
!!    pawfgrtab(natom)%rfgd(3,nfgd)= coordinates of r-r_atom around each atom
!!
!! PARENTS
!!      scfcv
!!
!! CHILDREN
!!      ext_buffers,ind_positions,leave_new,pawgylm,timab,wrtout,xredxcart
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

!PENDING: ADD PARALELLISM OVER ATOMS:
!COPY NHATGRID

subroutine wvl_nhatgrid(atindx1,geocode,h,i3s,natom,natom_tot,&
& nattyp,ntypat,n1,n1i,n2,n2i,n3,n3pi,optcut,optgr0,optgr1,optgr2,optrad,&
& pawfgrtab,pawtab,psppar,rprimd,shift,xred)


 use m_profiling

 use defs_basis
 use defs_datatypes
 use defs_abitypes
 use m_errors
! use m_xmpi, only : xcomm_rank,xcomm_size

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'wvl_nhatgrid'
 use interfaces_14_hidewrite
 use interfaces_16_hideleave
 use interfaces_18_timing
 use interfaces_66_paw, except_this_one => wvl_nhatgrid
!End of the abilint section

 implicit none

!Arguments ---------------------------------------------
!scalars
 integer,intent(in) :: i3s,natom,natom_tot,ntypat,optcut,optgr0,optgr1,optgr2,optrad
 integer,intent(in) ::n1,n2,n3,n1i,n2i,n3pi,shift
 real(dp),intent(in) :: h(3)
 character(1),intent(in)::geocode
! integer, intent(in),optional::mpi_comm_wvl
!arrays
 integer,intent(in) :: atindx1(natom),nattyp(ntypat)
 real(dp),intent(in) :: psppar(0:4,0:6,ntypat),rprimd(3,3)
 real(dp),intent(inout)::xred(3,natom)
 type(pawfgrtab_type),intent(inout) :: pawfgrtab(natom)
 type(pawtab_type),intent(in) :: pawtab(ntypat)

!Local variables ------------------------------
!scalars
!buffer to be added at the end of the last dimension of an array to control bounds_check
 integer :: iat,iatm,iatom,iatom_tot
 integer :: iex,iey,iez,ind,isx,isy,isz,itypat
 integer :: i1,i2,i3
 integer :: j1,j2,j3,lm_size
 integer :: nbl1,nbr1,nbl2,nbr2,nbl3,nbr3
 integer :: ncmax,nfgd
 real(dp) :: cutoff
 real(dp) :: rloc,rshp,rx,ry,rz,xcart(3,natom)
 real(dp) :: r2,r2shp
 real(dp) :: x,y,z
 real(dp) :: hh(3) !fine grid spacing for wavelets
 logical :: gox,goy,goz,perx,pery,perz
#if defined HAVE_DFT_BIGDFT
 !
#else
 character(len=500) :: message
#endif
!arrays
 integer,allocatable :: ifftsph_tmp(:)
 real(dp) :: tsec(2)
 real(dp),allocatable :: rr(:,:)

!debug
 !integer::itmp
!end debug

! *************************************************************************

 DBG_ENTER("COLL")

 call timab(559,1,tsec)

!Set up parallelism for wvl
!for debug: use me_wvl=xcomm_rank(MPI_COMM_WORLD)
!if (present(mpi_comm_wvl)) then
!me_wvl=xcomm_rank(mpi_comm_wvl)
!nproc_wvl=xcomm_size(mpi_comm_wvl)
!else
!me_wvl=0;nproc_wvl=1
!end if
!Pending: parallelism over atoms: see nhatgrid

 if (natom_tot<natom) then   ! This test has to be remove when natom_tot is used
   MSG_BUG(' natom_tot<natom !')
 end if

!Fine grid
 hh(:)=0.5d0*h(:)

!Compute xcart from xred
 call xredxcart(natom,1,rprimd,xcart,xred)

#if defined HAVE_DFT_BIGDFT

!Conditions for periodicity in the three directions
 perx=(geocode /= 'F')
 pery=(geocode == 'P')
 perz=(geocode /= 'F')

!Compute values of external buffers
 call ext_buffers(perx,nbl1,nbr1) 
 call ext_buffers(pery,nbl2,nbr2)
 call ext_buffers(perz,nbl3,nbr3)


!Loop over atoms
!
 iatm=0
 do itypat=1,ntypat
!  
   rloc=psppar(0,0,itypat)
   cutoff=10.d0*rloc
!  
   if (optcut==1) then
     rshp=pawtab(itypat)%rpaw
   else
     rshp=pawtab(itypat)%rshp
   end if
   r2shp=1.0000001_dp*rshp**2

!  allocate rr array
   if (n3pi > 0) then
!    use factor 1+int(1.1*, for safety reasons
     ncmax=1+int(( rshp/hh(1)+1.0)*(rshp/hh(2)+1.0)*(rshp/hh(3)+1.0)*four_pi/three)
   else
     ncmax=1
   end if
!  
   ABI_ALLOCATE(ifftsph_tmp,(ncmax))
   ABI_ALLOCATE(rr,(3,ncmax))
!  
   do iat=1,nattyp(itypat)
     iatm=iatm+1;iatom=atindx1(iatm)
     iatom_tot=iatom; !if (paral_atom) iatom_tot=my_atmtab(iatom)

!    Define a "box" around each atom
     rx=xcart(1,iatom_tot)
     ry=xcart(2,iatom_tot)
     rz=xcart(3,iatom_tot)
     
     isx=floor((rx-cutoff)/hh(1))
     isy=floor((ry-cutoff)/hh(2))
     isz=floor((rz-cutoff)/hh(3))
     
     iex=ceiling((rx+cutoff)/hh(1))
     iey=ceiling((ry+cutoff)/hh(2))
     iez=ceiling((rz+cutoff)/hh(3))
!    
     nfgd=0
     do i3=isz,iez
       z=real(i3,kind=8)*hh(3)-rz
       call ind_positions(perz,i3,n3,j3,goz)
       j3=j3+nbl3+1
       do i2=isy,iey
         y=real(i2,kind=8)*hh(2)-ry
         call ind_positions(pery,i2,n2,j2,goy)
         do i1=isx,iex
           x=real(i1,kind=8)*hh(1)-rx
           call ind_positions(perx,i1,n1,j1,gox)
           r2=x**2+y**2+z**2
           if (j3 >= i3s .and. j3 <= i3s+n3pi-1  .and. goy  .and. gox ) then
!            
             if(r2<=r2shp) then
               ind=j1+1+nbl1+(j2+nbl2)*n1i+(j3-i3s)*n1i*n2i
               nfgd=nfgd+1
!              test
!              itmp=400+me_wvl
!              write(itmp,'(i10,3(f13.7,x))')ind,x+rx,y+ry,z+rz
               rr(:,nfgd)=[x,y,z]
               ifftsph_tmp(nfgd)=shift+ind
             end if
           end if
         end do
       end do
     end do


     lm_size=pawfgrtab(iatom)%l_size**2

!    Allocate arrays defining sphere (and related data) around current atom
     if (associated(pawfgrtab(iatom)%ifftsph)) then
       ABI_DEALLOCATE(pawfgrtab(iatom)%ifftsph)
     end if
     ABI_ALLOCATE(pawfgrtab(iatom)%ifftsph,(nfgd))
     pawfgrtab(iatom)%nfgd=nfgd
     pawfgrtab(iatom)%ifftsph(1:nfgd)=ifftsph_tmp(1:nfgd)

     if (optrad==1) then
       if (associated(pawfgrtab(iatom)%rfgd)) then
         ABI_DEALLOCATE(pawfgrtab(iatom)%rfgd)
       end if
       ABI_ALLOCATE(pawfgrtab(iatom)%rfgd,(3,nfgd))
       pawfgrtab(iatom)%rfgd_allocated=1
       pawfgrtab(iatom)%rfgd(1:3,1:nfgd)=rr(1:3,1:nfgd)
     end if

     if (optgr0==1) then
       if (associated(pawfgrtab(iatom)%gylm)) then
         ABI_DEALLOCATE(pawfgrtab(iatom)%gylm)
       end if
       ABI_ALLOCATE(pawfgrtab(iatom)%gylm,(nfgd,lm_size))
       pawfgrtab(iatom)%gylm_allocated=1
     end if

     if (optgr1==1) then
       if (associated(pawfgrtab(iatom)%gylmgr)) then
         ABI_DEALLOCATE(pawfgrtab(iatom)%gylmgr)
       end if
       ABI_ALLOCATE(pawfgrtab(iatom)%gylmgr,(3,nfgd,lm_size))
       pawfgrtab(iatom)%gylmgr_allocated=1
     end if

     if (optgr2==1) then
       if (associated(pawfgrtab(iatom)%gylmgr2)) then
         ABI_DEALLOCATE(pawfgrtab(iatom)%gylmgr2)
       end if
       ABI_ALLOCATE(pawfgrtab(iatom)%gylmgr2,(6,nfgd,lm_size))
       pawfgrtab(iatom)%gylmgr2_allocated=1
     end if

!    ------------------------------------------------------------------
!    B-Calculate g_l(r)*Y_lm(r) for each r around the atom (if requested)
!    ------------------------------------------------------------------

     call pawgylm(pawfgrtab(iatom)%gylm,pawfgrtab(iatom)%gylmgr,pawfgrtab(iatom)%gylmgr2,&
&     lm_size,nfgd,optgr0,optgr1,optgr2,pawtab(itypat),rr(:,1:nfgd),1)

!    End loops over types/atoms
!    -------------------------------------------
!    
   end do
!  Deallocate arrays
   ABI_DEALLOCATE(ifftsph_tmp)
   ABI_DEALLOCATE(rr)
 end do

#else
 write(message, '(a,a,a,a)' ) ch10,&
& ' wvl_nhatgrid : BigDFT library is not compiled.', ch10, &
& '   Action, used the flag --with_dft_flavor="atompaw+.." when configuring.'
 call wrtout(std_out,message,'COLL')
 call leave_new('COLL')
#endif

 call timab(559,2,tsec)

 DBG_EXIT("COLL")

end subroutine wvl_nhatgrid


!!***
