!{\src2tex{textfont=tt}}
!!****f* ABINIT/outphbtrap
!! NAME
!! outphbtrap
!!
!! FUNCTION
!!  Print out phonon frequencies on regular grid for BoltzTrap
!!  Flag in input file is outboltztrap=1
!!
!! COPYRIGHT
!!  Copyright (C) 2006-2013 ABINIT group (MVer)
!!  This file is distributed under the terms of the
!!  GNU General Public License, see ~abinit/COPYING
!!  or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!   filnam = file name for output to disk
!!   nqpt = number of q-points
!!   dipdip= if 0, no dipole-dipole interaction was subtracted in atmfrc
!!           if 1, atmfrc has been build without dipole-dipole part
!!   symdynmat=if 1, (re)symmetrize the dynamical matrix, except if Gamma wavevector with electric field added.
!!   acell(3) =length scales by which rprim is to be multiplied
!!   amu(ntypat)=mass of the atoms (atomic mass unit)
!!   anaddb_dtset= (derived datatype) contains all the input variables
!!   atmfrc(2,3,natom,3,natom,nrpt) = Interatomic Forces in real space
!!   dielt(3,3)=dielectric tensor
!!   dyewq0(3,3,natom)=Ewald part of the dynamical matrix, at q=0.
!!   gmet(3,3)= metric tensor in reciprocal space.
!!   gprim(3,3)=normalized coordinates in reciprocal space
!!   indsym(4,nsym,natom)=label given by subroutine symatm, indicating atom
!!    label which gets rotated into given atom by given symmetry
!!    (first three elements are related primitive translation--
!!    see symatm where this is computed)
!!   mpert =maximum number of ipert
!!   msym = maximum number of symmetries
!!   natom=number of atoms in the unit cell
!!   nrpt=number of R points in the Big Box
!!   nsym=number of symmetries
!!   ntypat=number of atom types
!!   rmet(3,3)=metric tensor in real space.
!!   rprim(3,3)=dimensionless primitive translations in real space
!!   rpt(3,nprt)=canonical coordinates of the R points in the unit cell
!!    These coordinates are normalized (=> * acell(3)!!)
!!   symrec(3,3,nsym)=symmetry operations
!!   symrel(3,3,nsym)=symmetry operations
!!   trans(3,natom)=atomic translations : xred = rcan + trans
!!   typat(natom)=integer label of each type of atom (1,2,...)
!!   ucvol=unit cell volume
!!   wghatm(natom,natom,nrpt)=weights associated to a pair of atoms and to a R vector
!!   xred(3,natom)= relative coords of atoms in unit cell (dimensionless)
!!   zeff(3,3,natom)=effective charge on each atom, versus electric
!!    field and atomic displacement
!!
!! OUTPUT
!!  only write to file
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      anaddb
!!
!! CHILDREN
!!      gtdyn9,matr3inv,mkrdim,phfrq3,smpbz,symkpt,wrtout
!!
!! SOURCE

#if defined HAVE_CONFIG_H
#include "config.h"
#endif

#include "abi_common.h"

subroutine outphbtrap(filnam,dipdip,symdynmat,acell,amu,anaddb_dtset,&
& atmfrc,dielt,dyewq0,gmet,gprim,indsym,&
& mpert,msym,natom,nrpt,nsym,ntypat,rmet,rprim,rpt,symrec,symrel,trans,typat,ucvol,wghatm,xred,zeff)

 use defs_abitypes
 use defs_basis
 use m_errors
 use m_io_tools

!This section has been created automatically by the script Abilint (TD).
!Do not modify the following lines by hand.
#undef ABI_FUNC
#define ABI_FUNC 'outphbtrap'
 use interfaces_14_hidewrite
 use interfaces_32_util
 use interfaces_41_geometry
 use interfaces_56_recipspace
 use interfaces_72_response
 use interfaces_77_ddb, except_this_one => outphbtrap
!End of the abilint section

 implicit none

!Arguments -------------------------------
!scalars
 character(len=fnlen),intent(in) :: filnam

 integer,intent(in) :: dipdip,symdynmat
 integer,intent(in) :: mpert,msym,natom,nrpt,nsym,ntypat
 real(dp),intent(in) :: ucvol
 type(anaddb_dataset_type),intent(in) :: anaddb_dtset
!arrays
 integer,intent(in) :: indsym(4,nsym,natom),symrec(3,3,nsym),symrel(3,3,nsym),typat(natom)
 real(dp),intent(in) :: acell(3),amu(ntypat),dielt(3,3),gmet(3,3),gprim(3,3)
 real(dp),intent(in) :: rmet(3,3),rprim(3,3),rpt(3,nrpt)
 real(dp),intent(in) :: trans(3,natom),wghatm(natom,natom,nrpt),xred(3,natom)
 real(dp),intent(in) :: zeff(3,3,natom)
 real(dp),intent(inout) :: atmfrc(2,3,natom,3,natom,nrpt),dyewq0(3,3,natom)

!Local variables -------------------------
!scalars
 integer,parameter :: brav1=1,chksymbreak0=0
 integer :: facbrv,imode,iq_ibz
 integer :: nqbz,nqpt_max,nqshft,option,timrev
 integer :: nqibz, unit_btrap, iost
 integer :: iatom, jatom, idir, jdir
 real(dp) :: bzvol
 real(dp) :: qphnrm
 character(len=500) :: message
 character(len=fnlen) :: outfile
!arrays
 integer :: qptrlatt(3,3)
 integer,allocatable :: ibz2bz(:)
 real(dp) :: d2cart(2,3,mpert,3,mpert),displ(2*3*natom*3*natom),eigval(3*natom)
 real(dp) :: eigvec(2,3,natom,3*natom),gprimd(3,3),phfrq(3*natom)
 real(dp) :: qphon(3),rprimd(3,3)
 real(dp),allocatable :: qbz(:,:),qibz(:,:),qshft(:,:)
 real(dp),allocatable :: wtq(:),wtq_folded(:),wtqibz(:)

! *********************************************************************

 DBG_ENTER("COLL")


!*************************************************************************

 outfile = trim(filnam) // '_BTRAP'
 write(message, '(3a)')ch10,&
& ' Will write phonon FREQS in BoltzTrap format to file ',trim(outfile)
 call wrtout(ab_out,message,'COLL')
 call wrtout(std_out,message,'COLL')

 unit_btrap = get_unit()
 open (unit=unit_btrap,file=outfile,status='replace',iostat=iost)
 if (iost /= 0) then
   write (message,'(3a)')' Opening file ',trim(outfile),' as new'
   MSG_ERROR(message)
 end if

 write (unit_btrap,'(a)') '#'
 write (unit_btrap,'(a)') '# ABINIT package : Boltztrap phonon file. Remove this header before feeding to BT'
 write (unit_btrap,'(a)') '#    for compatibility with PHON output the freq are in Ry (before the square)'
 write (unit_btrap,'(a)') '#'
 write (unit_btrap,'(a)') '#    nq, nband  '
 write (unit_btrap,'(a)') '#  qx, qy, qz   '
 write (unit_btrap,'(a)') '#  qpt weight   '
 write (unit_btrap,'(a)') '#  freq_1^2, dynmat column for mode 1 '
 write (unit_btrap,'(a)') '#  etc for mode 2,3,4... qpt 2,3,4... '

 call mkrdim(acell,rprim,rprimd)
 call matr3inv(rprimd,gprimd)

 bzvol=ABS ( gprimd(1,1)*(gprimd(2,2)*gprimd(3,3)-gprimd(3,2)*gprimd(2,3)) &
& -gprimd(2,1)*(gprimd(1,2)*gprimd(3,3)-gprimd(3,2)*gprimd(1,3)) &
& +gprimd(3,1)*(gprimd(1,2)*gprimd(2,3)-gprimd(2,2)*gprimd(1,3)))
!
!Save memory during the generation of the q-mesh in the full BZ  
!Take into account the type of Bravais lattice
 facbrv=1
 if (brav1==2) facbrv=2
 if (brav1==3) facbrv=4

 nqshft=1 !always 1 
 ABI_ALLOCATE(qshft,(3,nqshft))
 qshft(:,1)=anaddb_dtset%q2shft(:)  ! FIXME small inconsistency in the dimension of q1shft

 nqpt_max=(anaddb_dtset%ng2qpt(1)*anaddb_dtset%ng2qpt(2)*anaddb_dtset%ng2qpt(3)*nqshft)/facbrv
 ABI_ALLOCATE(qibz,(3,nqpt_max))
 ABI_ALLOCATE(qbz,(3,nqpt_max))

 qptrlatt(:,:)=0
 qptrlatt(1,1)=anaddb_dtset%ng2qpt(1)
 qptrlatt(2,2)=anaddb_dtset%ng2qpt(2)
 qptrlatt(3,3)=anaddb_dtset%ng2qpt(3)
 option=1 
!
!here I noticed a problem in the declaration of q1shft in the anaddb datatype 
!FIXME we write on unit std_out just to avoid problem with automatic tests
 call smpbz(brav1,std_out,qptrlatt,nqpt_max,nqbz,nqshft,option,qshft,qbz)
!
!Reduce the number of such points by symmetrization.
 ABI_ALLOCATE(ibz2bz,(nqbz))
 ABI_ALLOCATE(wtq,(nqbz))
 ABI_ALLOCATE(wtq_folded,(nqbz))
 wtq(:)=one/nqbz         ! Weights sum up to one
 timrev=1; option=1     ! TODO timrev should be input 
!
 call symkpt(chksymbreak0,gmet,ibz2bz,std_out,qbz,nqbz,nqibz,nsym,symrec,timrev,wtq,wtq_folded)
 write(std_out,*) 'nqibz = ', nqibz

 ABI_ALLOCATE(wtqibz,(nqibz))
 do iq_ibz=1,nqibz
   wtqibz(iq_ibz)=wtq_folded(ibz2bz(iq_ibz))
   qibz(:,iq_ibz)=qbz(:,ibz2bz(iq_ibz))
 end do
 ABI_DEALLOCATE(wtq_folded)
 ABI_DEALLOCATE(qshft)

 write (unit_btrap,'(2I6)') nqibz, 3*natom 

 qphnrm=one
!=== Sum over irreducible q-points ===
 do iq_ibz=1,nqibz
   qphon(:)=qibz(:,iq_ibz);
!  
!  Get d2cart using interatomic forces and the long-range coulomb interaction through Ewald summation
   call gtdyn9(acell,atmfrc,dielt,dipdip,dyewq0,d2cart,gmet,gprim,&
&   mpert,natom,nrpt,qphnrm,qphon,rmet,rprim,rpt,trans,ucvol,wghatm,xred,zeff)
!  
!  Get eigenvectors and eigenvalues of the dynamical matrix, eigvec are normalized to one
   call phfrq3(amu,displ,d2cart,eigval,eigvec,indsym,mpert,msym,natom,nsym,ntypat,&
&   phfrq,qphnrm,qphon,rprimd,symdynmat,symrel,typat,ucvol)

   write (unit_btrap,'(3E20.10)') qphon
   write (unit_btrap,'(E20.10)') wtqibz(iq_ibz)
   do iatom = 1, natom
     do idir = 1, 3
       imode = idir + 3*(iatom-1)
!      factor two for Ry output - this may change in definitive BT and abinit formats 
       write (unit_btrap,'(E20.10)', ADVANCE='NO') phfrq(imode)*two
       do jatom = 1, natom
         do jdir = 1, 3
           write (unit_btrap,'(2E20.10)', ADVANCE='NO') d2cart(:,jdir,jatom,idir, iatom)
         end do
       end do
       write (unit_btrap,*)
     end do
   end do
   
 end do !irred q-points

 ABI_DEALLOCATE(ibz2bz)
 ABI_DEALLOCATE(qibz)
 ABI_DEALLOCATE(qbz)
 ABI_DEALLOCATE(wtq)
 ABI_DEALLOCATE(wtqibz)

 close (unit=unit_btrap)

 DBG_EXIT("COLL")

end subroutine outphbtrap
!!***
